@@ -15,6 +15,8 @@ from openstack.block_storage.v3 import availability_zone
 from openstack.block_storage.v3 import backup as _backup
 from openstack.block_storage.v3 import capabilities as _capabilities
 from openstack.block_storage.v3 import extension as _extension
+from openstack.block_storage.v3 import group as _group
+from openstack.block_storage.v3 import group_snapshot as _group_snapshot
 from openstack.block_storage.v3 import group_type as _group_type
 from openstack.block_storage.v3 import limits as _limits
 from openstack.block_storage.v3 import quota_set as _quota_set
@@ -464,7 +466,8 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
             when no resource can be found.
         """
         return self._find(_volume.Volume, name_or_id,
-                          ignore_missing=ignore_missing)
+                          ignore_missing=ignore_missing,
+                          list_base_path='/volumes/detail')
 
     def volumes(self, details=True, **query):
         """Retrieve a generator of volumes
@@ -517,6 +520,18 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
             volume = self._get_resource(_volume.Volume, volume)
             volume.force_delete(self)
 
+    def update_volume(self, volume, **attrs):
+        """Update a volume
+
+        :param volume: Either the ID of a volume or a
+            :class:`~openstack.block_storage.v3.volume.Volume` instance.
+        :param dict attrs: The attributes to update on the volume.
+
+        :returns: The updated volume
+        :rtype: :class:`~openstack.block_storage.v3.volume.Volume`
+        """
+        return self._update(_volume.Volume, volume, **attrs)
+
     def get_volume_metadata(self, volume):
         """Return a dictionary of metadata for a volume
 
@@ -977,6 +992,121 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         """
         return self._get(_capabilities.Capabilities, host)
 
+    # ====== GROUPS ======
+    def get_group(self, group_id, **attrs):
+        """Get a group
+
+        :param group_id: The ID of the group to get.
+        :param dict attrs:  Optional query parameters to be sent to limit the
+            resources being returned.
+
+        :returns: A Group instance.
+        :rtype: :class:`~openstack.block_storage.v3.group`
+        """
+        return self._get(_group.Group, group_id, **attrs)
+
+    def find_group(self, name_or_id, ignore_missing=True, **attrs):
+        """Find a single group
+
+        :param name_or_id: The name or ID of a group.
+        :param bool ignore_missing: When set to ``False``
+            :class:`~openstack.exceptions.ResourceNotFound` will be raised
+            when the group snapshot does not exist.
+
+        :returns: One :class:`~openstack.block_storage.v3.group.Group`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+            when no resource can be found.
+        """
+        return self._find(
+            _group.Group, name_or_id, ignore_missing=ignore_missing)
+
+    def groups(self, details=True, **query):
+        """Retrieve a generator of groups
+
+        :param bool details: When set to ``False``, no additional details will
+            be returned. The default, ``True``, will cause additional details
+            to be returned.
+        :param dict query: Optional query parameters to be sent to limit the
+            resources being returned:
+
+            * all_tenants: Shows details for all project.
+            * sort: Comma-separated list of sort keys and optional sort
+              directions.
+            * limit: Returns a number of items up to the limit value.
+            * offset: Used in conjunction with limit to return a slice of
+              items. Specifies where to start in the list.
+            * marker: The ID of the last-seen item.
+            * list_volume: Show volume ids in this group.
+            * detailed: If True, will list groups with details.
+            * search_opts: Search options.
+
+        :returns: A generator of group objects.
+        """
+        base_path = '/groups/detail' if details else '/groups'
+        return self._list(_group.Group, base_path=base_path, **query)
+
+    def create_group(self, **attrs):
+        """Create a new group from attributes
+
+        :param dict attrs: Keyword arguments which will be used to create
+            a :class:`~openstack.block_storage.v3.group.Group` comprised of
+            the properties on the Group class.
+
+        :returns: The results of group creation.
+        :rtype: :class:`~openstack.block_storage.v3.group.Group`.
+        """
+        return self._create(_group.Group, **attrs)
+
+    def create_group_from_source(self, **attrs):
+        """Creates a new group from source
+
+        :param dict attrs: Keyword arguments which will be used to create
+            a :class:`~openstack.block_storage.v3.group.Group` comprised of
+            the properties on the Group class.
+
+        :returns: The results of group creation.
+        :rtype: :class:`~openstack.block_storage.v3.group.Group`.
+        """
+        return _group.Group.create_from_source(self, **attrs)
+
+    def reset_group_state(self, group, status):
+        """Reset group status
+
+        :param group: The :class:`~openstack.block_storage.v3.group.Group`
+            to set the state.
+        :param status: The status for a group.
+
+        :returns: ``None``
+        """
+        res = self._get_resource(_group.Group, group)
+        return res.reset_status(self, status)
+
+    def delete_group(self, group, delete_volumes=False):
+        """Delete a group
+
+        :param group: The :class:`~openstack.block_storage.v3.group.Group` to
+            delete.
+        :param bool delete_volumes: When set to ``True``, volumes in group
+            will be deleted.
+
+        :returns: ``None``.
+        """
+        res = self._get_resource(_group.Group, group)
+        res.delete(self, delete_volumes=delete_volumes)
+
+    def update_group(self, group, **attrs):
+        """Update a group
+
+        :param group: The value can be the ID of a group or a
+            :class:`~openstack.block_storage.v3.group.Group` instance.
+        :param dict attrs: The attributes to update on the group.
+
+        :returns: The updated group
+        :rtype: :class:`~openstack.volume.v3.group.Group`
+        """
+        return self._update(_group.Group, group, **attrs)
+
+    # ====== AVAILABILITY ZONES ======
     def availability_zones(self):
         """Return a generator of availability zones
 
@@ -987,6 +1117,92 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
 
         return self._list(availability_zone.AvailabilityZone)
 
+    # ====== GROUP SNAPSHOT ======
+    def get_group_snapshot(self, group_snapshot_id):
+        """Get a group snapshot
+
+        :param group_snapshot_id: The ID of the group snapshot to get.
+
+        :returns: A GroupSnapshot instance.
+        :rtype: :class:`~openstack.block_storage.v3.group_snapshot`
+        """
+        return self._get(_group_snapshot.GroupSnapshot, group_snapshot_id)
+
+    def find_group_snapshot(self, name_or_id, ignore_missing=True):
+        """Find a single group snapshot
+
+        :param name_or_id: The name or ID of a group snapshot.
+        :param bool ignore_missing: When set to ``False``
+            :class:`~openstack.exceptions.ResourceNotFound` will be raised
+            when the group snapshot does not exist.
+
+        :returns: One :class:`~openstack.block_storage.v3.group_snapshot`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+            when no resource can be found.
+        """
+        return self._find(
+            _group_snapshot.GroupSnapshot, name_or_id,
+            ignore_missing=ignore_missing)
+
+    def group_snapshots(self, details=True, **query):
+        """Retrieve a generator of group snapshots
+
+        :param bool details: When ``True``, returns
+            :class:`~openstack.block_storage.v3.group_snapshot.GroupSnapshot`
+            objects with additional attributes filled.
+        :param kwargs query: Optional query parameters to be sent to limit
+            the group snapshots being returned.
+        :returns: A generator of group snapshtos.
+        """
+        base_path = '/group_snapshots'
+        if details:
+            base_path = '/group_snapshots/detail'
+
+        return self._list(
+            _group_snapshot.GroupSnapshot,
+            base_path=base_path,
+            **query,
+        )
+
+    def create_group_snapshot(self, **attrs):
+        """Create a group snapshot
+
+        :param dict attrs: Keyword arguments which will be used to create a
+            :class:`~openstack.block_storage.v3.group_snapshot.GroupSnapshot`
+            comprised of the properties on the GroupSnapshot class.
+
+        :returns: The results of group snapshot creation.
+        :rtype: :class:`~openstack.block_storage.v3.group_snapshot`.
+        """
+        return self._create(_group_snapshot.GroupSnapshot, **attrs)
+
+    def reset_group_snapshot_state(self, group_snapshot, state):
+        """Reset group snapshot status
+
+        :param group_snapshot: The
+            :class:`~openstack.block_storage.v3.group_snapshot.GroupSnapshot`
+            to set the state.
+        :param state: The state of the group snapshot to be set.
+
+        :returns: None
+        """
+        resource = self._get_resource(
+            _group_snapshot.GroupSnapshot, group_snapshot)
+        resource.reset_state(self, state)
+
+    def delete_group_snapshot(self, group_snapshot, ignore_missing=True):
+        """Delete a group snapshot
+
+        :param group_snapshot: The :class:`~openstack.block_storage.v3.
+            group_snapshot.GroupSnapshot` to delete.
+
+        :returns: None
+        """
+        self._delete(
+            _group_snapshot.GroupSnapshot, group_snapshot,
+            ignore_missing=ignore_missing)
+
+    # ====== GROUP TYPE ======
     def get_group_type(self, group_type):
         """Get a specific group type
 
@@ -1063,7 +1279,7 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
             When set to ``True``, no exception will be set when attempting to
             delete a nonexistent zone.
 
-        :returns: ''None''
+        :returns: None
         """
         self._delete(
             _group_type.GroupType, group_type, ignore_missing=ignore_missing)
@@ -1144,7 +1360,7 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
 
     # ====== QUOTA SETS ======
     def get_quota_set(self, project, usage=False, **query):
-        """Show quota set information for the project
+        """Show QuotaSet information for the project
 
         :param project: ID or instance of
             :class:`~openstack.identity.project.Project` of the project for
@@ -1164,7 +1380,7 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
             self, usage=usage, **query)
 
     def get_quota_set_defaults(self, project):
-        """Show quota set defaults for the project
+        """Show QuotaSet defaults for the project
 
         :param project: ID or instance of
             :class:`~openstack.identity.project.Project` of the project for
@@ -1181,7 +1397,7 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
             self, base_path='/os-quota-sets/defaults')
 
     def revert_quota_set(self, project, **query):
-        """Reset quota for the project/user.
+        """Reset Quota for the project/user.
 
         :param project: ID or instance of
             :class:`~openstack.identity.project.Project` of the project for
@@ -1194,17 +1410,16 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         res = self._get_resource(
             _quota_set.QuotaSet, None, project_id=project.id)
 
-        if not query:
-            query = {}
         return res.delete(self, **query)
 
     def update_quota_set(self, quota_set, query=None, **attrs):
-        """Update a quota set.
+        """Update a QuotaSet.
 
         :param quota_set: Either the ID of a quota_set or a
             :class:`~openstack.block_storage.v3.quota_set.QuotaSet` instance.
         :param dict query: Optional parameters to be used with update call.
-        :attrs kwargs: The attributes to update on the quota set
+        :param attrs: The attributes to update on the QuotaSet represented
+            by ``quota_set``.
 
         :returns: The updated QuotaSet
         :rtype: :class:`~openstack.block_storage.v3.quota_set.QuotaSet`
@@ -1232,16 +1447,16 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         return self._list(_extension.Extension)
 
     # ====== UTILS ======
-    def wait_for_status(self, res, status='available', failures=None,
-                        interval=2, wait=120):
+    def wait_for_status(
+        self, res, status='available', failures=None, interval=2, wait=120,
+    ):
         """Wait for a resource to be in a particular status.
 
         :param res: The resource to wait on to reach the specified status.
             The resource must have a ``status`` attribute.
         :type resource: A :class:`~openstack.resource.Resource` object.
-        :param status: Desired status.
-        :param failures: Statuses that would be interpreted as failures.
-        :type failures: :py:class:`list`
+        :param str status: Desired status.
+        :param list failures: Statuses that would be interpreted as failures.
         :param interval: Number of seconds to wait before to consecutive
             checks. Default to 2.
         :param wait: Maximum number of seconds to wait before the change.
@@ -1263,9 +1478,9 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
 
         :param res: The resource to wait on to be deleted.
         :type resource: A :class:`~openstack.resource.Resource` object.
-        :param interval: Number of seconds to wait before to consecutive
+        :param int interval: Number of seconds to wait before two consecutive
             checks. Default to 2.
-        :param wait: Maximum number of seconds to wait before the change.
+        :param int wait: Maximum number of seconds to wait before the change.
             Default to 120.
         :returns: The resource is returned on success.
         :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
@@ -1280,29 +1495,70 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
             }
         }
 
-    def _service_cleanup(self, dry_run=True, client_status_queue=None,
-                         identified_resources=None,
-                         filters=None, resource_evaluation_fn=None):
-        backups = []
-        for obj in self.backups(details=False):
-            need_delete = self._service_cleanup_del_res(
-                self.delete_backup,
-                obj,
-                dry_run=dry_run,
-                client_status_queue=client_status_queue,
-                identified_resources=identified_resources,
-                filters=filters,
-                resource_evaluation_fn=resource_evaluation_fn)
-            if not dry_run and need_delete:
-                backups.append(obj)
+    def _service_cleanup(
+        self,
+        dry_run=True,
+        client_status_queue=None,
+        identified_resources=None,
+        filters=None,
+        resource_evaluation_fn=None
+    ):
+        # It is not possible to delete backup if there are dependent backups.
+        # In order to be able to do cleanup those is required to have multiple
+        # iterations (first clean up backups with has no dependent backups, and
+        # in next iterations there should be no backups with dependencies
+        # remaining. Logically we can have also failures, therefore it is
+        # required to limit amount of iterations we do (currently pick 10).  In
+        # dry_run all those iterations are doing not what we want, therefore
+        # only iterate in a real cleanup mode.
+        if dry_run:
+            # Just iterate and evaluate backups in dry_run mode
+            for obj in self.backups(details=False):
+                need_delete = self._service_cleanup_del_res(
+                    self.delete_backup,
+                    obj,
+                    dry_run=dry_run,
+                    client_status_queue=client_status_queue,
+                    identified_resources=identified_resources,
+                    filters=filters,
+                    resource_evaluation_fn=resource_evaluation_fn)
+        else:
+            # Set initial iterations conditions
+            need_backup_iteration = True
+            max_iterations = 10
+            while need_backup_iteration and max_iterations > 0:
+                # Reset iteration controls
+                need_backup_iteration = False
+                max_iterations -= 1
+                backups = []
+                # To increase success chance sort backups by age, dependent
+                # backups are logically younger.
+                for obj in self.backups(
+                        details=True, sort_key='created_at', sort_dir='desc'
+                ):
+                    if not obj.has_dependent_backups:
+                        # If no dependent backups - go with it
+                        need_delete = self._service_cleanup_del_res(
+                            self.delete_backup,
+                            obj,
+                            dry_run=dry_run,
+                            client_status_queue=client_status_queue,
+                            identified_resources=identified_resources,
+                            filters=filters,
+                            resource_evaluation_fn=resource_evaluation_fn)
+                        if not dry_run and need_delete:
+                            backups.append(obj)
+                    else:
+                        # Otherwise we need another iteration
+                        need_backup_iteration = True
 
-        # Before deleting snapshots need to wait for backups to be deleted
-        for obj in backups:
-            try:
-                self.wait_for_delete(obj)
-            except exceptions.SDKException:
-                # Well, did our best, still try further
-                pass
+                # Before proceeding need to wait for backups to be deleted
+                for obj in backups:
+                    try:
+                        self.wait_for_delete(obj)
+                    except exceptions.SDKException:
+                        # Well, did our best, still try further
+                        pass
 
         snapshots = []
         for obj in self.snapshots(details=False):
