@@ -153,8 +153,7 @@ class ComputeCloudMixin(_normalize.Normalizer):
         """
         if not filters:
             filters = {}
-        return list(self.compute.keypairs(allow_unknown_params=True,
-                                          **filters))
+        return list(self.compute.keypairs(**filters))
 
     @_utils.cache_on_arguments()
     def list_availability_zone_names(self, unavailable=False):
@@ -353,7 +352,7 @@ class ComputeCloudMixin(_normalize.Normalizer):
         return [
             self._expand_server(server, detailed, bare)
             for server in self.compute.servers(
-                all_projects=all_projects, allow_unknown_params=True,
+                all_projects=all_projects,
                 **filters)
         ]
 
@@ -670,7 +669,7 @@ class ComputeCloudMixin(_normalize.Normalizer):
         return dict(server_vars=server_vars, groups=groups)
 
     @_utils.valid_kwargs(
-        'meta', 'files', 'userdata',
+        'meta', 'files', 'userdata', 'description',
         'reservation_id', 'return_raw', 'min_count',
         'max_count', 'security_groups', 'key_name',
         'availability_zone', 'block_device_mapping',
@@ -1097,7 +1096,8 @@ class ComputeCloudMixin(_normalize.Normalizer):
         nat_destination=None,
     ):
         if server['status'] == 'ERROR':
-            if 'fault' in server and 'message' in server['fault']:
+            if ('fault' in server and server['fault'] is not None
+                    and 'message' in server['fault']):
                 raise exc.OpenStackCloudException(
                     "Error in creating the server."
                     " Compute service reports fault: {reason}".format(
@@ -1504,7 +1504,6 @@ class ComputeCloudMixin(_normalize.Normalizer):
 
         return list(self.compute.hypervisors(
             details=True,
-            allow_unknown_params=True,
             **filters))
 
     def search_aggregates(self, name_or_id=None, filters=None):
@@ -1525,7 +1524,7 @@ class ComputeCloudMixin(_normalize.Normalizer):
 
         :returns: A list of compute ``Aggregate`` objects.
         """
-        return self.compute.aggregates(allow_unknown_params=True, **filters)
+        return self.compute.aggregates(**filters)
 
     # TODO(stephenfin): This shouldn't return a munch
     def get_aggregate(self, name_or_id, filters=None):
