@@ -82,8 +82,8 @@ class FloatingIPCloudMixin(_normalize.Normalizer):
     def _neutron_list_floating_ips(self, filters=None):
         if not filters:
             filters = {}
-        data = self.network.get('/floatingips', params=filters)
-        return self._get_and_munchify('floatingips', data)
+        data = list(self.network.ips(**filters))
+        return data
 
     def _nova_list_floating_ips(self):
         try:
@@ -102,10 +102,10 @@ class FloatingIPCloudMixin(_normalize.Normalizer):
             of this dictionary may, themselves, be dictionaries. Example::
 
                 {
-                  'last_name': 'Smith',
-                  'other': {
-                      'gender': 'Female'
-                  }
+                    'last_name': 'Smith',
+                    'other': {
+                        'gender': 'Female'
+                    }
                 }
 
             OR
@@ -113,7 +113,7 @@ class FloatingIPCloudMixin(_normalize.Normalizer):
             Example:: "[?last_name==`Smith`] | [?other.gender]==`Female`]"
 
         :returns: A floating IP ``munch.Munch`` or None if no matching floating
-                  IP is found.
+            IP is found.
 
         """
         return _utils._get_entity(self, 'floating_ip', id, filters)
@@ -228,11 +228,8 @@ class FloatingIPCloudMixin(_normalize.Normalizer):
         error_message = "Error getting floating ip with ID {id}".format(id=id)
 
         if self._use_neutron_floating():
-            data = proxy._json_response(
-                self.network.get('/floatingips/{id}'.format(id=id)),
-                error_message=error_message)
-            return self._normalize_floating_ip(
-                self._get_and_munchify('floatingip', data))
+            fip = self.network.get_ip(id)
+            return self._normalize_floating_ip(fip)
         else:
             data = proxy._json_response(
                 self.compute.get('/os-floating-ips/{id}'.format(id=id)),
@@ -253,7 +250,7 @@ class FloatingIPCloudMixin(_normalize.Normalizer):
         :returns: a list of floating IP addresses.
 
         :raises: ``OpenStackCloudResourceNotFound``, if an external network
-                 that meets the specified criteria cannot be found.
+            that meets the specified criteria cannot be found.
         """
         if project_id is None:
             # Make sure we are only listing floatingIPs allocated the current
@@ -312,7 +309,7 @@ class FloatingIPCloudMixin(_normalize.Normalizer):
         :returns: a list of floating IP addresses.
 
         :raises: ``OpenStackCloudResourceNotFound``, if a floating IP pool
-                 is not specified and cannot be found.
+            is not specified and cannot be found.
         """
 
         with _utils.shade_exceptions(
@@ -461,10 +458,8 @@ class FloatingIPCloudMixin(_normalize.Normalizer):
 
     def _submit_create_fip(self, kwargs):
         # Split into a method to aid in test mocking
-        data = self.network.post(
-            "/floatingips", json={"floatingip": kwargs})
-        return self._normalize_floating_ip(
-            self._get_and_munchify('floatingip', data))
+        data = self.network.create_ip(**kwargs)
+        return self._normalize_floating_ip(data)
 
     def _neutron_create_floating_ip(
             self, network_name_or_id=None, server=None,
@@ -474,8 +469,9 @@ class FloatingIPCloudMixin(_normalize.Normalizer):
 
         if not network_id:
             if network_name_or_id:
-                network = self.get_network(network_name_or_id)
-                if not network:
+                try:
+                    network = self.network.find_network(network_name_or_id)
+                except exceptions.ResourceNotFound:
                     raise exc.OpenStackCloudResourceNotFound(
                         "unable to find network for floating ips with ID "
                         "{0}".format(network_name_or_id))
@@ -612,15 +608,11 @@ class FloatingIPCloudMixin(_normalize.Normalizer):
 
     def _neutron_delete_floating_ip(self, floating_ip_id):
         try:
-            proxy._json_response(self.network.delete(
-                "/floatingips/{fip_id}".format(fip_id=floating_ip_id),
-                error_message="unable to delete floating IP"))
-        except exc.OpenStackCloudResourceNotFound:
+            self.network.delete_ip(
+                floating_ip_id, ignore_missing=False
+            )
+        except exceptions.ResourceNotFound:
             return False
-        except Exception as e:
-            raise exc.OpenStackCloudException(
-                "Unable to delete floating IP ID {fip_id}: {msg}".format(
-                    fip_id=floating_ip_id, msg=str(e)))
         return True
 
     def _nova_delete_floating_ip(self, floating_ip_id):
@@ -751,14 +743,9 @@ class FloatingIPCloudMixin(_normalize.Normalizer):
         if fixed_address is not None:
             floating_ip_args['fixed_ip_address'] = fixed_address
 
-        return proxy._json_response(
-            self.network.put(
-                "/floatingips/{fip_id}".format(fip_id=floating_ip['id']),
-                json={'floatingip': floating_ip_args}),
-            error_message=("Error attaching IP {ip} to "
-                           "server {server_id}".format(
-                               ip=floating_ip['id'],
-                               server_id=server['id'])))
+        return self.network.update_ip(
+            floating_ip,
+            **floating_ip_args)
 
     def _nova_attach_ip_to_server(self, server_id, floating_ip_id,
                                   fixed_address=None):
@@ -809,13 +796,16 @@ class FloatingIPCloudMixin(_normalize.Normalizer):
         f_ip = self.get_floating_ip(id=floating_ip_id)
         if f_ip is None or not f_ip['attached']:
             return False
-        exceptions.raise_from_response(
-            self.network.put(
-                "/floatingips/{fip_id}".format(fip_id=floating_ip_id),
-                json={"floatingip": {"port_id": None}}),
-            error_message=("Error detaching IP {ip} from "
-                           "server {server_id}".format(
-                               ip=floating_ip_id, server_id=server_id)))
+        try:
+            self.network.update_ip(
+                floating_ip_id,
+                port_id=None
+            )
+        except exceptions.SDKException:
+            raise exceptions.SDKException(
+                ("Error detaching IP {ip} from "
+                 "server {server_id}".format(
+                     ip=floating_ip_id, server_id=server_id)))
 
         return True
 
@@ -866,6 +856,7 @@ class FloatingIPCloudMixin(_normalize.Normalizer):
             f_ip = self.create_floating_ip(
                 server=server,
                 network=network, nat_destination=nat_destination,
+                fixed_address=fixed_address,
                 wait=wait, timeout=timeout)
             timeout = timeout - (time.time() - start_time)
             # Wait for cache invalidation time so that we don't try
@@ -900,15 +891,17 @@ class FloatingIPCloudMixin(_normalize.Normalizer):
 
         :raises: ``OpenStackCloudException``, on operation error.
         """
-        if type(ips) == list:
-            ip = ips[0]
-        else:
-            ip = ips
-        f_ip = self.get_floating_ip(
-            id=None, filters={'floating_ip_address': ip})
-        return self._attach_ip_to_server(
-            server=server, floating_ip=f_ip, wait=wait, timeout=timeout,
-            fixed_address=fixed_address)
+
+        if type(ips) != list:
+            ips = [ips]
+
+        for ip in ips:
+            f_ip = self.get_floating_ip(
+                id=None, filters={'floating_ip_address': ip})
+            server = self._attach_ip_to_server(
+                server=server, floating_ip=f_ip, wait=wait, timeout=timeout,
+                fixed_address=fixed_address)
+        return server
 
     def add_auto_ip(self, server, wait=False, timeout=60, reuse=True):
         """Add a floating IP to a server.
@@ -1005,34 +998,47 @@ class FloatingIPCloudMixin(_normalize.Normalizer):
     def _needs_floating_ip(self, server, nat_destination):
         """Figure out if auto_ip should add a floating ip to this server.
 
-        If the server has a public_v4 it does not need a floating ip.
+        If the server has a floating ip it does not need another one.
 
-        If the server does not have a private_v4 it does not need a
+        If the server does not have a fixed ip address it does not need a
         floating ip.
 
         If self.private then the server does not need a floating ip.
 
-        If the cloud runs nova, and the server has a private_v4 and not
-        a public_v4, then the server needs a floating ip.
+        If the cloud runs nova, and the server has a private address and not a
+        public address, then the server needs a floating ip.
 
-        If the server has a private_v4 and no public_v4 and the cloud has
-        a network from which floating IPs come that is connected via a
-        router to the network from which the private_v4 address came,
+        If the server has a fixed ip address and no floating ip address and the
+        cloud has a network from which floating IPs come that is connected via
+        a router to the network from which the fixed ip address came,
         then the server needs a floating ip.
 
-        If the server has a private_v4 and no public_v4 and the cloud
-        does not have a network from which floating ips come, or it has
+        If the server has a fixed ip address and no floating ip address and the
+        cloud does not have a network from which floating ips come, or it has
         one but that network is not connected to the network from which
-        the server's private_v4 address came via a router, then the
+        the server's fixed ip address came via a router, then the
         server does not need a floating ip.
         """
         if not self._has_floating_ips():
             return False
 
-        if server['public_v4']:
+        if server['addresses'] is None:
+            # fetch missing server details, e.g. because
+            # meta.add_server_interfaces() was not called
+            server = self.compute.get_server(server)
+
+        if server['public_v4'] \
+           or any([any([address['OS-EXT-IPS:type'] == 'floating'
+                        for address in addresses])
+                   for addresses
+                   in (server['addresses'] or {}).values()]):
             return False
 
-        if not server['private_v4']:
+        if not server['private_v4'] \
+           and not any([any([address['OS-EXT-IPS:type'] == 'fixed'
+                             for address in addresses])
+                        for addresses
+                        in (server['addresses'] or {}).values()]):
             return False
 
         if self.private:
