@@ -19,6 +19,8 @@ from openstack import exceptions
 from openstack.image.v2 import _proxy
 from openstack.image.v2 import image
 from openstack.image.v2 import member
+from openstack.image.v2 import metadef_namespace
+from openstack.image.v2 import metadef_schema
 from openstack.image.v2 import schema
 from openstack.image.v2 import service_info as si
 from openstack.image.v2 import task
@@ -45,6 +47,8 @@ class TestImageProxy(test_proxy_base.TestProxyBase):
         self.proxy = _proxy.Proxy(self.session)
         self.proxy._connection = self.cloud
 
+
+class TestImage(TestImageProxy):
     def test_image_import_no_required_attrs(self):
         # container_format and disk_format are required attrs of the image
         existing_image = image.Image(id="id")
@@ -194,6 +198,28 @@ class TestImageProxy(test_proxy_base.TestProxyBase):
             container_format='fake_cformat', disk_format='fake_dformat',
             name='fake', properties=mock.ANY)
 
+    def test_image_create_protected(self):
+        self.proxy.find_image = mock.Mock()
+
+        created_image = mock.Mock(spec=image.Image(id="id"))
+        self.proxy._create = mock.Mock()
+        self.proxy._create.return_value = created_image
+        self.proxy._create.return_value.image_import_methods = []
+
+        created_image.upload = mock.Mock()
+        created_image.upload.return_value = FakeResponse(response="",
+                                                         status_code=200)
+
+        properties = {"is_protected": True}
+
+        self.proxy.create_image(
+            name="fake", data="data", container_format="bare",
+            disk_format="raw", **properties
+        )
+
+        args, kwargs = self.proxy._create.call_args
+        self.assertEqual(kwargs["is_protected"], True)
+
     def test_image_upload_no_args(self):
         # container_format and disk_format are required args
         self.assertRaises(exceptions.InvalidRequest, self.proxy.upload_image)
@@ -321,6 +347,8 @@ class TestImageProxy(test_proxy_base.TestProxyBase):
             method_args=["image"],
             expected_args=[self.proxy])
 
+
+class TestMember(TestImageProxy):
     def test_member_create(self):
         self.verify_create(self.proxy.add_member, member.Member,
                            method_kwargs={"image": "test_id"},
@@ -381,6 +409,52 @@ class TestImageProxy(test_proxy_base.TestProxyBase):
                          method_kwargs={'image': 'image_1'},
                          expected_kwargs={'image_id': 'image_1'})
 
+
+class TestMetadefNamespace(TestImageProxy):
+    def test_metadef_namespace_create(self):
+        self.verify_create(
+            self.proxy.create_metadef_namespace,
+            metadef_namespace.MetadefNamespace,
+        )
+
+    def test_metadef_namespace_delete(self):
+        self.verify_delete(
+            self.proxy.delete_metadef_namespace,
+            metadef_namespace.MetadefNamespace,
+            False,
+        )
+
+    def test_metadef_namespace_delete__ignore(self):
+        self.verify_delete(
+            self.proxy.delete_metadef_namespace,
+            metadef_namespace.MetadefNamespace,
+            True,
+        )
+
+    def test_metadef_namespace_get(self):
+        self.verify_get(
+            self.proxy.get_metadef_namespace,
+            metadef_namespace.MetadefNamespace,
+        )
+
+    def test_metadef_namespaces(self):
+        self.verify_list(
+            self.proxy.metadef_namespaces,
+            metadef_namespace.MetadefNamespace,
+        )
+
+    def test_metadef_namespace_update(self):
+        # we're (intentionally) adding an additional field, 'namespace', to the
+        # request body
+        self.verify_update(
+            self.proxy.update_metadef_namespace,
+            metadef_namespace.MetadefNamespace,
+            method_kwargs={'is_protected': True},
+            expected_kwargs={'namespace': 'resource_id', 'is_protected': True},
+        )
+
+
+class TestSchema(TestImageProxy):
     def test_images_schema_get(self):
         self._verify(
             "openstack.proxy.Proxy._get",
@@ -413,6 +487,8 @@ class TestImageProxy(test_proxy_base.TestProxyBase):
             expected_kwargs={
                 'base_path': '/schemas/member', 'requires_id': False})
 
+
+class TestTask(TestImageProxy):
     def test_task_get(self):
         self.verify_get(self.proxy.get_task, task.Task)
 
@@ -429,7 +505,7 @@ class TestImageProxy(test_proxy_base.TestProxyBase):
         result = self.proxy.wait_for_task(
             res, status, "failure", 0.01, 0.1)
 
-        self.assertTrue(result, res)
+        self.assertEqual(res, result)
 
     def test_wait_for_task_immediate_status_case(self):
         status = "SUCcess"
@@ -438,7 +514,7 @@ class TestImageProxy(test_proxy_base.TestProxyBase):
         result = self.proxy.wait_for_task(
             res, status, "failure", 0.01, 0.1)
 
-        self.assertTrue(result, res)
+        self.assertEqual(res, result)
 
     def test_wait_for_task_error_396(self):
         # Ensure we create a new task when we get 396 error
@@ -510,6 +586,8 @@ class TestImageProxy(test_proxy_base.TestProxyBase):
             expected_kwargs={
                 'base_path': '/schemas/task', 'requires_id': False})
 
+
+class TestMisc(TestImageProxy):
     def test_stores(self):
         self.verify_list(self.proxy.stores, si.Store)
 
@@ -521,3 +599,95 @@ class TestImageProxy(test_proxy_base.TestProxyBase):
             method_kwargs={},
             expected_args=[si.Import],
             expected_kwargs={'require_id': False})
+
+
+class TestMetadefSchema(TestImageProxy):
+    def test_metadef_namespace_schema_get(self):
+        self._verify(
+            "openstack.proxy.Proxy._get",
+            self.proxy.get_metadef_namespace_schema,
+            expected_args=[metadef_schema.MetadefSchema],
+            expected_kwargs={
+                'base_path': '/schemas/metadefs/namespace',
+                'requires_id': False})
+
+    def test_metadef_namespaces_schema_get(self):
+        self._verify(
+            "openstack.proxy.Proxy._get",
+            self.proxy.get_metadef_namespaces_schema,
+            expected_args=[metadef_schema.MetadefSchema],
+            expected_kwargs={
+                'base_path': '/schemas/metadefs/namespaces',
+                'requires_id': False})
+
+    def test_metadef_resource_type_schema_get(self):
+        self._verify(
+            "openstack.proxy.Proxy._get",
+            self.proxy.get_metadef_resource_type_schema,
+            expected_args=[metadef_schema.MetadefSchema],
+            expected_kwargs={
+                'base_path': '/schemas/metadefs/resource_type',
+                'requires_id': False})
+
+    def test_metadef_resource_types_schema_get(self):
+        self._verify(
+            "openstack.proxy.Proxy._get",
+            self.proxy.get_metadef_resource_types_schema,
+            expected_args=[metadef_schema.MetadefSchema],
+            expected_kwargs={
+                'base_path': '/schemas/metadefs/resource_types',
+                'requires_id': False})
+
+    def test_metadef_object_schema_get(self):
+        self._verify(
+            "openstack.proxy.Proxy._get",
+            self.proxy.get_metadef_object_schema,
+            expected_args=[metadef_schema.MetadefSchema],
+            expected_kwargs={
+                'base_path': '/schemas/metadefs/object',
+                'requires_id': False})
+
+    def test_metadef_objects_schema_get(self):
+        self._verify(
+            "openstack.proxy.Proxy._get",
+            self.proxy.get_metadef_objects_schema,
+            expected_args=[metadef_schema.MetadefSchema],
+            expected_kwargs={
+                'base_path': '/schemas/metadefs/objects',
+                'requires_id': False})
+
+    def test_metadef_property_schema_get(self):
+        self._verify(
+            "openstack.proxy.Proxy._get",
+            self.proxy.get_metadef_property_schema,
+            expected_args=[metadef_schema.MetadefSchema],
+            expected_kwargs={
+                'base_path': '/schemas/metadefs/property',
+                'requires_id': False})
+
+    def test_metadef_properties_schema_get(self):
+        self._verify(
+            "openstack.proxy.Proxy._get",
+            self.proxy.get_metadef_properties_schema,
+            expected_args=[metadef_schema.MetadefSchema],
+            expected_kwargs={
+                'base_path': '/schemas/metadefs/properties',
+                'requires_id': False})
+
+    def test_metadef_tag_schema_get(self):
+        self._verify(
+            "openstack.proxy.Proxy._get",
+            self.proxy.get_metadef_tag_schema,
+            expected_args=[metadef_schema.MetadefSchema],
+            expected_kwargs={
+                'base_path': '/schemas/metadefs/tag',
+                'requires_id': False})
+
+    def test_metadef_tags_schema_get(self):
+        self._verify(
+            "openstack.proxy.Proxy._get",
+            self.proxy.get_metadef_tags_schema,
+            expected_args=[metadef_schema.MetadefSchema],
+            expected_kwargs={
+                'base_path': '/schemas/metadefs/tags',
+                'requires_id': False})
